// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

package frc.robot;

import com.pathplanner.lib.auto.AutoBuilder;
import com.pathplanner.lib.auto.NamedCommands;

import edu.wpi.first.math.MathUtil;
import edu.wpi.first.math.geometry.Pose2d;
import edu.wpi.first.wpilibj.smartdashboard.SendableChooser;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.button.CommandXboxController;
import frc.robot.constants.OIConstants;
import frc.robot.subsystems.Drivetrain;

public class RobotContainer {
  private Drivetrain drivetrain;

  private CommandXboxController primary;
  private CommandXboxController secondary;

  private SendableChooser<Command> autoChooser;

  public RobotContainer() {
    // TODO Need to provide a real initial pose
    // TODO Need to provide a real VisualPoseProvider, null means we're not using one at all
    drivetrain = new Drivetrain(new Pose2d(), null);

    // An example Named Command, doesn't need to remain once we start actually adding real things
    // ALL Named Commands need to be defined AFTER subsystem initialization and BEFORE auto/controller configuration
    NamedCommands.registerCommand("Set Drivetrain X", drivetrain.setXCommand());

    // TODO Specify a default auto string once we have one
    autoChooser = AutoBuilder.buildAutoChooser();

    primary = new CommandXboxController(OIConstants.kPrimaryXboxUSB);
    secondary = new CommandXboxController(OIConstants.kSecondaryXboxUSB);

    configureBindings();
  }

  // The objective should be to have the subsystems expose methods that return commands
  // that can be bound to the triggers provided by the CommandXboxController class.
  // This mindset should help keep RobotContainer a little cleaner this year. 
  // This is not to say you can't trigger or command chain here (see driveCardnial drivetrain example),
  // but generally avoid any situation where the keyword "new" is involved
  private void configureBindings() {
    drivetrain.setDefaultCommand(drivetrain.teleopCommand(
      primary::getLeftX, 
      primary::getLeftY, 
      primary::getRightX, 
      OIConstants.kTeleopDriveDeadband
    ));

    primary.povCenter().onFalse(
      drivetrain.driveCardinal(
        primary::getLeftX,
        primary::getLeftY,
        primary.getHID()::getPOV,
        OIConstants.kTeleopDriveDeadband).until(
          () -> MathUtil.applyDeadband(primary.getRightX(), OIConstants.kTeleopDriveDeadband) != 0
        )
    );

    // This was originally a run while held, not sure that's really necessary
    primary.y().onTrue(drivetrain.zeroHeadingCommand());

    // This was originally a run while held, not sure that's really necessary
    primary.rightBumper().onTrue(drivetrain.setXCommand());
  }

  public Command getAutonomousCommand() {
    return autoChooser.getSelected();
  }
}
